import datetime
import logging
import os
from typing import Dict, List, Optional

import boto3
from pydantic import BaseModel

from acryl_datahub_cloud.datahub_reporting.datahub_dataset import (
    DataHubBasedS3Dataset,
    DatasetMetadata,
    DatasetRegistrationSpec,
    FileStoreBackedDatasetConfig,
)
from acryl_datahub_cloud.datahub_reporting.datahub_form_reporting import (
    DataHubFormReportingData,
)
from acryl_datahub_cloud.datahub_reporting.forms_config import (
    DataHubReportingFormSourceConfig,
    DataHubReportingFormSourceReport,
)
from datahub.emitter.mce_builder import make_dataset_urn
from datahub.ingestion.api.common import PipelineContext
from datahub.ingestion.api.decorators import (
    SupportStatus,
    config_class,
    platform_name,
    support_status,
)
from datahub.ingestion.api.source import Source, SourceReport
from datahub.ingestion.graph.client import DataHubGraph

logger = logging.getLogger(__name__)


class FormAnalyticsConfig(BaseModel):
    enabled: bool
    dataset_urn: Optional[str]
    physical_uri_prefix: Optional[str]


@platform_name(id="datahub", platform_name="DataHub")
@config_class(DataHubReportingFormSourceConfig)
@support_status(SupportStatus.INCUBATING)
class DataHubReportingFormsSource(Source):
    platform = "datahub"

    def __init__(self, config: DataHubReportingFormSourceConfig, ctx: PipelineContext):
        super().__init__(ctx)
        self.config: DataHubReportingFormSourceConfig = config
        self.report = DataHubReportingFormSourceReport()
        self.opened_files: List[str] = []
        self.s3_client = boto3.client("s3")

    def get_reporting_config(self) -> Optional[FormAnalyticsConfig]:
        query_name = "formAnalyticsConfig"
        field_mappings: Dict[str, str] = {
            "enabled": "enabled",
            "dataset_urn": "datasetUrn",
            "physical_uri_prefix": "physicalUriPrefix",
        }
        query_project_fragment = "\n".join(field_mappings.values())
        form_config_query = f"""
            query {{
                {query_name} {{
                    {query_project_fragment}
                 }}
            }}
            """

        query_result = self.graph.execute_graphql(query=form_config_query)
        if query_result:
            if query_result.get(query_name, {}).get("enabled") is False:
                return FormAnalyticsConfig(enabled=False)
            result_map = query_result.get(query_name, {})
            return FormAnalyticsConfig.parse_obj(
                dict(
                    (field, result_map.get(graphql_field))
                    for field, graphql_field in field_mappings.items()
                )
            )
        else:
            return None

    def get_workunits(self):
        self.graph = (
            self.ctx.require_graph("Loading default graph coordinates.")
            if self.config.server is None
            else DataHubGraph(config=self.config.server)
        )
        form_analytics_config = self.get_reporting_config()

        if form_analytics_config and not form_analytics_config.enabled:
            logger.error("Form analytics is not enabled. Skipping reporting.")
            self.report.feature_enabled = False
            return

        form_data = DataHubFormReportingData(self.graph, self.config.forms_include)
        # If form analytics config is not present, use the default reporting bucket prefix
        dataset_uri_prefix = (
            form_analytics_config.physical_uri_prefix
            if form_analytics_config and form_analytics_config.physical_uri_prefix
            else self.config.reporting_bucket_prefix
        )
        if not dataset_uri_prefix:
            raise ValueError(
                "Reporting bucket prefix must be provided. Either configure it on the server side or in the source config."
            )

        registration_spec = DatasetRegistrationSpec()
        dataset_metadata = DatasetMetadata(
            displayName="Forms Reporting Data",
            description="This data was generated by the forms reporting system.",
        )
        dataset_urn = (
            form_analytics_config.dataset_urn
            if form_analytics_config and form_analytics_config.dataset_urn
            else make_dataset_urn(
                self.config.reporting_store_platform, self.config.reporting_dataset_name
            )
        )
        dataset_config = FileStoreBackedDatasetConfig(
            dataset_name="Forms Reporting Dataset",
            store_platform="s3",
            dataset_registration_spec=registration_spec,
            bucket_prefix=dataset_uri_prefix,
            dataset_urn=dataset_urn,
        )
        dataset = DataHubBasedS3Dataset(
            dataset_metadata=dataset_metadata,
            config=dataset_config,
        )
        for row in form_data.get_data(
            lambda x: self.report.increment_assets_scanned(),
            lambda x: self.report.increment_forms_scanned(),
        ):
            dataset.append(row)

        for mcp in dataset.commit():
            assert mcp.entityUrn, "MCP must have a URN"
            dataset_urn = mcp.entityUrn
            yield mcp.as_workunit()
        logger.info(
            f"Reporting file created at {dataset.get_remote_file_uri(dataset_uri_prefix=dataset_uri_prefix, date=datetime.date.today())}"
        )
        logger.info(f"Reporting dataset registered at {dataset_urn}")

        print(f"Dataset {dataset_urn} created successfully")

    def get_report(self) -> SourceReport:
        return self.report

    def close(self) -> None:
        for file in self.opened_files:
            os.remove(file)
        return super().close()



import json
import logging
from datetime import date, datetime
from enum import Enum
from typing import Any, Callable, Dict, Iterable, List, Optional

import pandas as pd
from pydantic import BaseModel

from acryl_datahub_cloud.datahub_reporting.datahub_dataset import BaseModelRow
from datahub.emitter.mcp import MetadataChangeProposalWrapper
from datahub.ingestion.graph.client import DataHubGraph
from datahub.ingestion.graph.filters import SearchFilterRule
from datahub.metadata.schema_classes import (
    DomainPropertiesClass,
    FormInfoClass,
    FormsClass,
    FormTypeClass,
)

logger = logging.getLogger(__name__)


class FormStatus(str, Enum):
    NOT_STARTED = "not_started"
    IN_PROGRESS = "in_progress"
    COMPLETED = "complete"


class QuestionStatus(str, Enum):
    NOT_STARTED = "not_started"
    COMPLETED = "complete"


class FormType(str, Enum):
    DOCUMENTATION = "documentation"
    VERIFICATION = "verification"


class FormReportingRow(BaseModelRow):
    form_urn: str
    form_type: FormType
    form_assigned_date: date
    form_completed_date: Optional[date]
    form_status: FormStatus
    question_id: str
    question_status: QuestionStatus
    question_completed_date: Optional[date]
    assignee_urn: Optional[str]
    asset_urn: str
    platform_urn: str
    platform_instance_urn: str
    domain_urn: Optional[str]
    parent_domain_urn: Optional[str]
    asset_verified: Optional[bool]
    snapshot_date: date


class FormData:
    def get_data(
        self,
        on_asset_scanned: Optional[Callable[[str], Any]] = None,
        on_form_scanned: Optional[Callable[[str], Any]] = None,
    ) -> Iterable[FormReportingRow]:
        raise NotImplementedError

    def to_mcps(self) -> Iterable[MetadataChangeProposalWrapper]:
        raise NotImplementedError


class FormRegistry:
    def __init__(self, graph: DataHubGraph):
        self.registry: Dict[str, Optional[FormInfoClass]] = {}
        self.graph = graph

    def get_form(self, form_id: str) -> Optional[FormInfoClass]:
        if form_id not in self.registry:
            self._load_form(form_id)
        return self.registry[form_id]

    def _load_form(self, form_id: str) -> None:
        form_urn = (
            f"urn:li:form:{form_id}"
            if not form_id.startswith("urn:li:form:")
            else form_id
        )

        form_info = self.graph.get_aspect(form_urn, FormInfoClass)
        self.registry[form_id] = form_info


class DomainRegistry:
    def __init__(self, graph: DataHubGraph):
        self.registry: Dict[str, Optional[DomainPropertiesClass]] = {}
        self.graph = graph

    def get_domain(self, domain_id: str) -> Optional[DomainPropertiesClass]:
        if domain_id not in self.registry:
            self._load_domain(domain_id)
        return self.registry[domain_id]

    def _load_domain(self, domain_id: str) -> None:
        domain_urn = (
            f"urn:li:domain:{domain_id}"
            if not domain_id.startswith("urn:li:domain:")
            else domain_id
        )
        domain_props = self.graph.get_aspect(domain_urn, DomainPropertiesClass)
        self.registry[domain_id] = domain_props


class DataHubFormReportingData(FormData):
    class DataHubDatasetSearchRow(BaseModel):
        urn: str
        owners: List[str] = []
        completedForms: List[str] = []
        incompleteForms: List[str] = []
        completedFormsIncompletePromptIds: List[str] = []
        completedFormsIncompletePromptResponseTimes: List[str] = []
        completedFormsCompletedPromptIds: List[str] = []
        completedFormsCompletedPromptResponseTimes: List[str] = []
        incompleteFormsIncompletePromptIds: List[str] = []
        incompleteFormsIncompletePromptResponseTimes: List[str] = []
        incompleteFormsCompletedPromptIds: List[str] = []
        incompleteFormsCompletedPromptResponseTimes: List[str] = []
        platform: str = ""
        platformInstance: str = ""
        domains: List[str] = []

    def __init__(self, graph: DataHubGraph, allowed_forms: Optional[List[str]] = None):
        self.graph: DataHubGraph = graph
        self.form_registry = FormRegistry(graph)
        self.domain_registry = DomainRegistry(graph)
        self.snapshot_date = datetime.now().date()
        self.allowed_forms = allowed_forms

    def get_dataframe(
        self,
        on_asset_scanned: Callable[[str], Any],
        on_form_scanned: Callable[[str], Any],
    ) -> pd.DataFrame:
        return pd.DataFrame(
            x.dict()
            for x in self.get_data(
                on_asset_scanned=on_asset_scanned, on_form_scanned=on_form_scanned
            )
        )

    def get_form_existence_or_filters(self) -> List[SearchFilterRule]:
        """
        Datasets must either have completedForms or incompleteForms assigned to
        them
        """
        if self.allowed_forms:
            return [
                {
                    "field": "completedForms",
                    "condition": "EQUAL",
                    "values": self.allowed_forms,
                },
                {
                    "field": "incompleteForms",
                    "condition": "EQUAL",
                    "values": self.allowed_forms,
                },
            ]
        else:
            return [
                {
                    "field": "completedForms",
                    "condition": "EXISTS",
                },
                {
                    "field": "incompleteForms",
                    "condition": "EXISTS",
                },
            ]

    def form_assigned_date(
        self, search_row: DataHubDatasetSearchRow
    ) -> Dict[str, date]:

        form_assigned_dates: Dict[str, date] = {}
        forms = self.graph.get_aspect(search_row.urn, FormsClass)
        if not forms:
            return form_assigned_dates
        assert forms, f"Forms aspect not found for {search_row.urn}"
        for incomplete_form in forms.incompleteForms:
            form_assigned_dates[incomplete_form.urn] = datetime.fromtimestamp(
                incomplete_form.created.time / 1000 if incomplete_form.created else 0
            ).date()
        for completed_form in forms.completedForms:
            form_assigned_dates[completed_form.urn] = datetime.fromtimestamp(
                completed_form.created.time / 1000 if completed_form.created else 0
            ).date()
        return form_assigned_dates

    def form_completed_date(
        self, search_row: DataHubDatasetSearchRow
    ) -> Dict[str, date]:
        # get the maximum timestamp from the
        # incompleteFormsCompletedPromptResponseTimes and
        # completedFormsCompletedPromptResponseTimes
        form_completion_dates = {}
        for form in search_row.completedForms:
            form_info = self.form_registry.get_form(form)
            assert form_info, f"Form {form} not found"
            form_prompts = [x.id for x in form_info.prompts]
            completed_prompts_map = {
                prompt_id: response_time
                for prompt_id, response_time in zip(
                    search_row.completedFormsCompletedPromptIds,
                    search_row.completedFormsCompletedPromptResponseTimes,
                )
                if prompt_id in form_prompts
            }
            sorted_dates = sorted(completed_prompts_map.values())
            if sorted_dates:
                form_completion_dates[form] = datetime.fromtimestamp(
                    int(sorted(completed_prompts_map.values())[-1]) / 1000
                ).date()
            else:
                logger.warning(
                    f"Form {form} on asset {search_row.urn} has no completed prompts but is marked as completed"
                )
        return form_completion_dates

    def get_assignees(self, form_info: FormInfoClass, owners: List[str]) -> List[str]:
        assignees = []
        actors = form_info.actors
        if actors.owners:
            # form applies to owners
            assignees.extend(owners)
        if actors.users:
            assignees.extend(actors.users)
        if actors.groups:
            assignees.extend(actors.groups)
        return list(set(assignees))

    def get_data(  # noqa: C901
        self,
        on_asset_scanned: Optional[Callable[[str], Any]] = None,
        on_form_scanned: Optional[Callable[[str], Any]] = None,
    ) -> Iterable[FormReportingRow]:
        extra_fields = [f for f in self.DataHubDatasetSearchRow.__fields__.keys()]
        result = self.graph.get_results_by_filter(
            entity_types=["dataset"],
            extra_or_filters=self.get_form_existence_or_filters(),
            extra_source_fields=extra_fields,
            skip_cache=True,
        )
        forms_scanned = set()
        row_index = 0
        for row in result:
            row_index += 1
            if row_index % 100 == 0:
                logger.info(f"Scanned {row_index} assets")
            extra_properties = row["extraProperties"]

            extra_properties_map = {
                x["name"]: json.loads(x["value"]) for x in extra_properties
            }
            search_row = self.DataHubDatasetSearchRow(**extra_properties_map)
            if on_asset_scanned:
                on_asset_scanned(search_row.urn)
            form_assigned_dates = self.form_assigned_date(search_row)
            form_completed_dates = self.form_completed_date(search_row)
            domain = None
            parent_domain = None
            if search_row.domains:
                domain = search_row.domains[0]
                domain_props = self.domain_registry.get_domain(domain)
                if not domain_props:
                    logger.warning(
                        f"Domain {search_row.domains[0]} not found. Will record the domain urn regardless"
                    )
                else:
                    if domain_props.parentDomain:
                        parent_domain = domain_props.parentDomain
            # for owner in owners:
            incomplete_forms = (
                [x for x in search_row.incompleteForms if x in self.allowed_forms]
                if self.allowed_forms
                else search_row.incompleteForms
            )
            for form_id in incomplete_forms:
                if form_id not in forms_scanned:
                    if on_form_scanned:
                        on_form_scanned(form_id)
                    forms_scanned.add(form_id)
                form_info = self.form_registry.get_form(form_id)
                assert form_info, f"Form {form_id} not found"
                form_prompts = [x.id for x in form_info.prompts]
                form_incomplete_prompts = [
                    p
                    for p in search_row.incompleteFormsIncompletePromptIds
                    if p in form_prompts
                ]
                form_status = (
                    FormStatus.IN_PROGRESS
                    if len(form_incomplete_prompts) < len(form_prompts)
                    else FormStatus.NOT_STARTED
                )
                form_type = (
                    FormType.DOCUMENTATION
                    if form_info.type == FormTypeClass.COMPLETION
                    else FormType.VERIFICATION
                )
                assignees = self.get_assignees(form_info, search_row.owners)
                for prompt_id in [
                    p
                    for p in search_row.incompleteFormsIncompletePromptIds
                    if p in form_prompts
                ]:
                    for owner in assignees:
                        yield FormReportingRow(
                            form_urn=form_id,
                            form_assigned_date=form_assigned_dates[form_id],
                            form_completed_date=None,
                            form_status=form_status,
                            form_type=form_type,
                            assignee_urn=owner,
                            asset_urn=search_row.urn,
                            platform_urn=search_row.platform,
                            platform_instance_urn=search_row.platformInstance,
                            domain_urn=domain,
                            parent_domain_urn=parent_domain,
                            asset_verified=(
                                False if form_type == FormType.VERIFICATION else None
                            ),
                            question_id=str(prompt_id),
                            question_status=QuestionStatus.NOT_STARTED,
                            question_completed_date=None,
                            snapshot_date=self.snapshot_date,
                        )
                for prompt_id, prompt_reponse_time in [
                    (p, p_response_time)
                    for (p, p_response_time) in zip(
                        search_row.incompleteFormsCompletedPromptIds,
                        search_row.incompleteFormsCompletedPromptResponseTimes,
                    )
                    if p in form_prompts
                ]:
                    for owner in assignees:
                        yield FormReportingRow(
                            form_urn=form_id,
                            form_assigned_date=form_assigned_dates[form_id],
                            form_completed_date=None,
                            form_status=form_status,
                            form_type=form_type,
                            assignee_urn=owner,
                            asset_urn=search_row.urn,
                            platform_urn=search_row.platform,
                            platform_instance_urn=search_row.platformInstance,
                            domain_urn=domain,
                            parent_domain_urn=parent_domain,
                            asset_verified=(
                                False if form_type == FormType.VERIFICATION else None
                            ),
                            question_id=str(prompt_id),
                            question_status=QuestionStatus.COMPLETED,
                            question_completed_date=prompt_reponse_time,
                            snapshot_date=self.snapshot_date,
                        )
            complete_forms = (
                [x for x in search_row.completedForms if x in self.allowed_forms]
                if self.allowed_forms
                else search_row.completedForms
            )
            for form_id in complete_forms:
                if form_id not in forms_scanned:
                    if on_form_scanned:
                        on_form_scanned(form_id)
                    forms_scanned.add(form_id)
                form_info = self.form_registry.get_form(form_id)
                assert form_info, f"Form {form_id} not found"
                form_type = (
                    FormType.DOCUMENTATION
                    if form_info.type == FormTypeClass.COMPLETION
                    else FormType.VERIFICATION
                )
                assignees = self.get_assignees(form_info, search_row.owners)
                form_prompts = [x.id for x in form_info.prompts]
                for prompt_id in [
                    p
                    for p in search_row.completedFormsIncompletePromptIds
                    for p in form_prompts
                ]:
                    logger.warning("Unexpected incomplete prompt in completed form")
                    for owner in assignees:
                        yield FormReportingRow(
                            form_urn=form_id,
                            form_assigned_date=form_assigned_dates[form_id],
                            form_completed_date=form_completed_dates[form_id],
                            form_status=FormStatus.COMPLETED,
                            form_type=form_type,
                            assignee_urn=owner,
                            asset_urn=search_row.urn,
                            platform_urn=search_row.platform,
                            platform_instance_urn=search_row.platformInstance,
                            domain_urn=domain,
                            parent_domain_urn=parent_domain,
                            asset_verified=(
                                True if form_type == FormType.VERIFICATION else None
                            ),
                            question_id=str(prompt_id),
                            question_status=QuestionStatus.NOT_STARTED,
                            question_completed_date=None,
                            snapshot_date=self.snapshot_date,
                        )
                for prompt_id, prompt_reponse_time in [
                    (p, p_response_time)
                    for (p, p_response_time) in zip(
                        search_row.completedFormsCompletedPromptIds,
                        search_row.completedFormsCompletedPromptResponseTimes,
                    )
                    if p in form_prompts
                ]:
                    for owner in assignees:
                        yield FormReportingRow(
                            form_urn=form_id,
                            form_assigned_date=form_assigned_dates[form_id],
                            form_completed_date=form_completed_dates[form_id],
                            form_status=FormStatus.COMPLETED,
                            form_type=form_type,
                            assignee_urn=owner,
                            asset_urn=search_row.urn,
                            platform_urn=search_row.platform,
                            platform_instance_urn=search_row.platformInstance,
                            domain_urn=domain,
                            parent_domain_urn=parent_domain,
                            asset_verified=(
                                True if form_type == FormType.VERIFICATION else None
                            ),
                            question_id=str(prompt_id),
                            question_status=QuestionStatus.COMPLETED,
                            question_completed_date=prompt_reponse_time,
                            snapshot_date=self.snapshot_date,
                        )


if __name__ == "__main__":
    from datahub.ingestion.graph.client import get_default_graph

    with get_default_graph() as graph:
        form_data = DataHubFormReportingData(graph)
        for row in form_data.get_data():
            print(row)



import logging
import os
import shutil
import zipfile
from datetime import datetime, timedelta
from pathlib import Path
from typing import Iterable, List, Optional

import boto3
from pydantic import validator

from acryl_datahub_cloud.datahub_reporting.datahub_dataset import (
    BaseModelRow,
    DataHubBasedS3Dataset,
    DatasetMetadata,
    DatasetRegistrationSpec,
    FileStoreBackedDatasetConfig,
)
from datahub.configuration.common import ConfigModel
from datahub.emitter.mcp import MetadataChangeProposalWrapper
from datahub.ingestion.api.common import PipelineContext
from datahub.ingestion.api.decorators import (
    SupportStatus,
    config_class,
    platform_name,
    support_status,
)
from datahub.ingestion.api.source import Source, SourceReport
from datahub.ingestion.graph.client import DatahubClientConfig, DataHubGraph
from datahub.metadata.schema_classes import DatasetPropertiesClass

logger = logging.getLogger(__name__)


class S3ClientConfig(ConfigModel):
    bucket: str = os.getenv("DATA_BUCKET", "")
    path: str = os.getenv("RDS_DATA_PATH", "rds_backup/metadata_aspect_v2")


class DataHubReportingExtractSQLSourceConfig(ConfigModel):
    server: Optional[DatahubClientConfig] = None
    sql_backup_config: S3ClientConfig
    extract_sql_store: FileStoreBackedDatasetConfig

    @validator("extract_sql_store", pre=True, always=True)
    def set_default_extract_soft_delete_flag(cls, v):
        if v is not None:
            if "dataset_registration_spec" not in v:
                v["dataset_registration_spec"] = DatasetRegistrationSpec(
                    soft_deleted=False
                )
            elif "soft_deleted" not in v["dataset_registration_spec"]:
                v["dataset_registration_spec"]["soft_deleted"] = False

            if "file" not in v:
                default_config = FileStoreBackedDatasetConfig.dummy()
                v[
                    "file"
                ] = f"{default_config.file_name}.{default_config.file_extension}"
            else:
                v["file_name"] = v["file"].split(".")[0]
                v["file_extension"] = v["file"].split(".")[-1]
        return v


class SQLGraphRow(BaseModelRow):
    urn: str
    aspect: str
    version: int
    metadata: str
    systemmetadata: Optional[str]
    createdon: float
    createdby: str
    createdfor: str


@platform_name(id="datahub", platform_name="DataHub")
@config_class(DataHubReportingExtractSQLSourceConfig)
@support_status(SupportStatus.INCUBATING)
class DataHubReportingExtractSQLSource(Source):
    platform = "datahub"

    def __init__(
        self, config: DataHubReportingExtractSQLSourceConfig, ctx: PipelineContext
    ):
        super().__init__(ctx)
        self.config: DataHubReportingExtractSQLSourceConfig = config
        self.report = SourceReport()
        self.opened_files: List[str] = []
        self.s3_client = boto3.client("s3")
        self.datahub_based_s3_dataset = DataHubBasedS3Dataset(
            self.config.extract_sql_store,
            dataset_metadata=DatasetMetadata(
                displayName="SQL Extract",
                description="This is an automated SQL Extract from DataHub's backend",
                schemaFields=SQLGraphRow.datahub_schema(),
            ),
        )

    @staticmethod
    def should_skip_extract(
        dataset_properties: Optional[DatasetPropertiesClass],
    ) -> bool:
        """Skip graph extraction if the dataset has already been pushed to DataHub today"""

        skip_extract = False
        # If present check is the updated timestamp for the current day
        if dataset_properties and dataset_properties.lastModified:
            # datetime function requires timestamp in seconds
            ts = datetime.fromtimestamp(dataset_properties.lastModified.time / 1000)
            skip_extract = ts.day is datetime.today().day

            if skip_extract:
                logger.info(
                    f"Skipping graph extract as dataset has been updated today {ts}"
                )

        return skip_extract

    def get_workunits(self):

        self.graph = (
            self.ctx.require_graph("Loading default graph coordinates.")
            if self.config.server is None
            else DataHubGraph(config=self.config.server)
        )

        dataset_properties: Optional[DatasetPropertiesClass] = self.graph.get_aspect(
            self.datahub_based_s3_dataset.get_dataset_urn(), DatasetPropertiesClass
        )

        # Generate a static path used for temporary files,
        # so we consistently delete the data (possibly based on pipeline name)
        tmp_dir_aux = (
            self.ctx.pipeline_name if self.ctx.pipeline_name else "sql_default_dir"
        )
        tmp_dir = f'/tmp/{tmp_dir_aux.replace(":", "_")}'

        output_file = (
            self.datahub_based_s3_dataset.config.file
            if self.datahub_based_s3_dataset.config.file
            else f"{self.datahub_based_s3_dataset.config.file_name}.zip"
        )

        mcps: Iterable[MetadataChangeProposalWrapper] = []
        if (
            self.should_skip_extract(dataset_properties)
            and self.datahub_based_s3_dataset.config.generate_presigned_url
        ):
            mcps = self.datahub_based_s3_dataset.update_presigned_url(
                dataset_properties=dataset_properties
            )
        else:
            # Get yesterday's RDS data dump.
            previous_date = datetime.now() - timedelta(days=1)
            time_partition_path = "year={}/month={:02d}/day={:02d}".format(
                previous_date.year, previous_date.month, previous_date.day
            )

            self._clean_up_old_state(state_directory=tmp_dir)
            self._download_files(
                bucket=self.config.sql_backup_config.bucket,
                prefix=f"{self.config.sql_backup_config.path}/{time_partition_path}",
                target_dir=f"{tmp_dir}/download/",
            )
            self._zip_folder(
                folder_path=f"{tmp_dir}/download",
                output_file=f"{tmp_dir}/{output_file}",
            )

            # Compute profile & schema information, this is based on the parquet files that were downloaded and not the zip file.
            # We must hard-code the local file from which the dataset will be created, otherwise the upload to s3 will be in
            # unexpected path.
            mcps = self.datahub_based_s3_dataset.register_dataset(
                dataset_urn=self.datahub_based_s3_dataset.get_dataset_urn(),
                physical_uri=self.datahub_based_s3_dataset.get_file_uri(),
                local_file=f"{tmp_dir}/download/*.parquet",
            )

            # Force update the DataHubBasedS3Dataset local file path to match where the zip file was created.
            self.datahub_based_s3_dataset.local_file_path = f"{tmp_dir}/{output_file}"
            # Upload zip file to s3
            self.datahub_based_s3_dataset.upload_file_to_s3()

        for mcp in mcps:
            # logger.warning(json.dumps(str(mcp)))
            yield mcp.as_workunit()

        logger.info(
            f"Reporting dataset registered at {self.datahub_based_s3_dataset.get_dataset_urn()}"
        )

        self._clean_up_old_state(state_directory=tmp_dir)

    @staticmethod
    def _clean_up_old_state(state_directory: str) -> None:
        shutil.rmtree(state_directory, ignore_errors=True)
        path = Path(f"{state_directory}/download/")
        path.mkdir(parents=True, exist_ok=True)

    def _download_files(self, bucket: str, prefix: str, target_dir: str) -> None:
        objects = self.s3_client.list_objects_v2(Bucket=bucket, Prefix=prefix)
        if "Contents" in objects:
            # Iterate over objects in the time partition path
            for obj in objects["Contents"]:
                # Extract file key
                file_key = obj["Key"]

                # Generate local file path
                local_file_path = os.path.join(
                    os.getcwd(), target_dir, os.path.basename(file_key)
                )

                # Download file from S3
                self.s3_client.download_file(bucket, file_key, local_file_path)
                # print(f"Downloaded {file_key} to {local_file_path}")
        else:
            logger.warning(f"No objects found in {prefix}")

    @staticmethod
    def _zip_folder(folder_path: str, output_file: str) -> None:
        with zipfile.ZipFile(output_file, "x", zipfile.ZIP_DEFLATED) as zipf:
            for root, _, files in os.walk(folder_path):
                for file in files:
                    file_path = os.path.join(root, file)
                    # Add file to zip archive with relative path
                    zipf.write(file_path, os.path.relpath(file_path, folder_path))

    def _upload_file(self, bucket: str, prefix: str, file: str) -> None:
        self.s3_client.upload_file(file, bucket, prefix)

    def get_report(self) -> SourceReport:
        return self.report

    def close(self) -> None:
        for file in self.opened_files:
            os.remove(file)
        return super().close()



import json
import pathlib

from setuptools import setup

_codegen_config_file = pathlib.Path("./src/acryl_datahub_cloud/_codegen_config.json")
_codegen_config: dict = json.loads(_codegen_config_file.read_text())

# Adding pydantic<2 since we use pydantic models to map to pyarrow models and that is only compatible in pydantic v1
stats_common = {"pandas", "pyarrow", "duckdb", "pydantic<2"}
aws_common = {"boto3"}

plugins = {
    "datahub-reporting-forms": stats_common | aws_common,
    "datahub-reporting-extract-graph": stats_common
    | aws_common
    | {"opensearch-py==2.4.2"},
    "datahub-reporting-extract-sql": stats_common | aws_common,
}

dev_requirements = {
    # acryl-datahub[dev] pulls in more things than are strictly necessary, but it's fine.
    "acryl-datahub[dev]",
    *list(
        dependency
        for plugin in [
            "datahub-reporting-forms",
            "datahub-reporting-extract-graph",
            "datahub-reporting-extract-sql"
        ]
        for dependency in plugins[plugin]
    ),
}

setup(
    **{
        **_codegen_config,
        "install_requires": [
            *_codegen_config["install_requires"],
        ],
        "entry_points": {
            **_codegen_config["entry_points"],
            "console_scripts": [
                "acryl-datahub-cloud = acryl_datahub_cloud.cli:main",
            ],
            "datahub.ingestion.source.plugins": [
                "datahub-reporting-forms = acryl_datahub_cloud.datahub_reporting.forms:DataHubReportingFormsSource",
                "datahub-reporting-extract-graph = acryl_datahub_cloud.datahub_reporting.extract_graph:DataHubReportingExtractGraphSource",
                "datahub-reporting-extract-sql = acryl_datahub_cloud.datahub_reporting.extract_sql:DataHubReportingExtractSQLSource",
            ],
        },
    },
    extras_require={
        **{plugin: list(dependencies) for (plugin, dependencies) in plugins.items()},
        "all": list(
            set().union(*[requirements for _plugin, requirements in plugins.items()])
        ),
        "dev": list(dev_requirements),
    }
)
